/////////////////
//

#ifdef _WIN32
#include "stdafx.h"
#endif

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <BipCpi.h>
#include <BipCci.h>
#include "ParserUtils.h"

extern void Log(char *szFormat, ...);

///////////////////////////////////////////////////////////
// mqsixRegisterMessages
#ifdef _WIN32
void mqsixRegisterMessages(HMODULE hModule, char *messageSet)
{
	char	strFileName[512];
	char	strRegistryPath[512];
	HKEY	key;
	DWORD	iDisposition;
	DWORD	iValue;
	DWORD	iRc;

	GetModuleFileName(hModule, strFileName, sizeof(strFileName));
	Log("File name of DLL: %s\n", strFileName);
	sprintf(strRegistryPath,
		"SYSTEM\\CurrentControlSet\\Services\\Eventlog\\Application\\%s", messageSet);
	
	iRc = RegCreateKeyEx(HKEY_LOCAL_MACHINE,
		strRegistryPath,
		0,
		"",
		REG_OPTION_NON_VOLATILE,
		KEY_WRITE,
		NULL,
		&key,
		&iDisposition);
	if (iRc != ERROR_SUCCESS)
	{
		Log("Failed to RegCreateKeyEx: %d\n", iRc);
		return;
	}
	if (iDisposition == REG_CREATED_NEW_KEY)
	{
		RegSetValueEx(key, "EventMessageFile", 0, REG_SZ, (PBYTE)strFileName, strlen(strFileName) + 1);
		iValue = 7;
		RegSetValueEx(key, "TypesSupported", 0, REG_DWORD, (PBYTE)&iValue, sizeof(iValue));
	}
	RegCloseKey(key);
}
#endif

///////////////////////////////////////////////////////////
// ASCIIToUnicode
CciChar* ASCIIToUnicode(const char* aStr)
{
	int	iLength = sizeof(CciChar) * (strlen(aStr) + 1);
	CciChar	*ucsString = (CciChar *)malloc(iLength);
	int	iRc;
	cciMbsToUcs(
		&iRc,
		aStr,
		ucsString,
		iLength,
		1208);
	return(ucsString);
}

///////////////////////////////////////////////////////////
// UnicodeToASCII
char *UnicodeToASCII(CciChar* ucsString)
{
	int	iRc;
	// WARNING!!:  The wcslen() function used in the following statement
	// is not believed to be portable.
	int iLength = wcslen(ucsString) + 1;
	char *mbString = (char *)malloc(iLength);
	cciUcsToMbs(
		&iRc,
		ucsString,
		mbString,
		iLength,
		1208);
	return(mbString);
}

///////////////////////////////////////////////////////////
// GetName
char *GetName(CciParser *parser, CciElement *element)
{
	int	iRc;
	CciChar	name[256];
	cpiElementName(&iRc, element, name, sizeof(name));
	if (iRc != 0)
	{
		return(0);
	}
	return(UnicodeToASCII(name));
}


///////////////////////////////////////////////////////////
// GetStringValue
char *GetStringValue(struct TreeAsIsContext *pTreeAsIsContext, CciParser *parser, CciElement *element)
{
	int	iRc;
	CciSize iSize;
	iSize = cpiElementCharacterValue(&iRc, element, pTreeAsIsContext->pData, pTreeAsIsContext->iSize);
	if (iRc == CCI_BUFFER_TOO_SMALL)
	{
		Log("Buffer too small for value, resizing to %d\n", iSize);
		free(pTreeAsIsContext->pData);
		pTreeAsIsContext->iSize = iSize;
		pTreeAsIsContext->pData = (CciChar *)malloc(sizeof(CciChar) * iSize + 1);
		iSize = cpiElementCharacterValue(&iRc, element, pTreeAsIsContext->pData, pTreeAsIsContext->iSize);
	}
	if (iRc != 0)
	{
		Log("Failed to get character value: rc=%d\n", iRc);
		return(0);
	}
	return(UnicodeToASCII(pTreeAsIsContext->pData));
}
